class BoxCoaterEventsController < ApplicationController
  before_action :block_index_all, only: [:index]

  model_data_store(BoxCoaterEvent)

  def apply_select
    [BoxCoaterEvent.arel_table[Arel.star], BoxCoaterRecipe.arel_table['processing_time']]
  end

  def set_target
    @model = model_type.select(apply_select)
                       .joins("LEFT OUTER JOIN box_coater_recipes ON box_coater_recipes.recipe_code = box_coater_events.recipe")
                       .find(params[:id])
  end

  def before_set_targets
    unless params[:filtered].nil?
      params[:filtered].each do |filter|
        if filter[:id] == 'time'
          start_time = resolve_time_filter(filter[:value])
          return @models.joins("LEFT OUTER JOIN box_coater_recipes ON box_coater_recipes.recipe_code = box_coater_events.recipe")
                        .where(BoxCoaterEvent.arel_table[:time].gteq(start_time))
        end
      end
    end
  end

  # Because the params are passed in from Rx-EER, the filters are a stringified hash
  def before_filter(filter)
    return nil if filter[:id] == 'time'
    if filter[:id] == 'batch_id'
      filter[:id] = 'box_coater_batch.batch_id'
    end
    filter
  end

  def update
    params = model_params
    begin
      # Run as a transaction so if either the event or the batch fails to update, the other is reversed
      BoxCoaterEvent.transaction do
        @model.update!(operator_note: params[:operator_note], recipe: params[:recipe], reporting_category: params[:reporting_category])

        unless params[:batch].nil?
          batch_info = params[:batch]
          @batch = BoxCoaterBatch.find(batch_info[:id])
          @batch.update!(lens_count: batch_info[:lens_count], empty_holes_count: batch_info[:empty_holes_count]) unless @batch.nil?
        end
      end
      render :show
    rescue ActiveRecord::RecordInvalid
      render status: :unprocessable_entity
    end
  end

  def latest_for_machines
    params = latest_for_machine_params

    @models = []

    # Todo : can this be optimized to run a single query?
    params[:machine_ids].each do |machine_id|
      event = BoxCoaterEvent.where(machine_id: machine_id).order(time: :desc).first
      @models << event unless event.nil?
    end

    render "box_coater_events/index"
  end

  private

  def resolve_time_filter(filter_name)
    return Time.now.at_beginning_of_day if filter_name == 'day'
    return Time.now.at_beginning_of_week if filter_name == 'week'
    return Time.now.at_beginning_of_month if filter_name == 'month'
    return Time.now
  end

  # Only allow a list of trusted parameters through.
  def model_params
    params.permit(:operator_note, :recipe, :reporting_category, batch: {})
  end

  def latest_for_machine_params
    params.permit(machine_ids: [])
  end
end
