class EventsController < ApplicationController
  include MachineResponse
  include MachineDataViewFilter
  include RxModule::Tableable

  def self.set_target_before
    super + [:event_data]
  end

  wrap_parameters :event, include: [:message, :type, :eventData, :date, :time, :sn]
  model_data_store(Event)

  # get /event/1.0/list
  # params:
  #   startTime
  #   endTime
  #   offset
  #   limit
  #   resourceSn
  #   resourceGroupName
  # TODO: implement resourceGroupName
  def mes_events
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @models = Event.where(time: start_time..end_time)
      @models = @models.where(machine_sn: params[:resourceSn]) if params[:resourceSn]
      @models = @models.offset(@offset).limit(@limit)
      render :events
    end
  end

  def add
    begin
      AddEventJob.perform_later(model_params, events)
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  def event_data
    render :event_data
  end

  def before_set_targets
    models = if filters["sn"]
      Event.where(machine_sn: filters["sn"])
    else
      Event
    end
    view_before_set_targets(models, params)
  end

  def before_filter(filter)
    view_before_filter(filter)
  end

  private

  # Never trust parameters from the scary internet, only allow the white list through.
  def model_params
    event = params.require(:event)
    {
      message: event[:message],
      event_type: event[:type],
      time: "#{params[:date]} #{params[:time]}",
      machine_sn: params[:sn]
    }
  end

  def events
    params.require(:event).permit(eventData: {data: [:name, :value]}).dig(:eventData)&.dig(:data) || []
  end
end
