class MachinesController < ApplicationController
  include MachineBoxCoater

  def self.set_target_before
    super + [:archive, :restore, :production_data, :machine_last_state] + MachineBoxCoater::box_coater_set_target_before
  end

  wrap_parameters :machine, include: [:sn]
  model_data_store(Machine, {create: true, read: true, update: true, delete: false, index: true}, {create: true, read: true, update: true, delete: false, index: true})
  protect_with('Machine', "Machine", [RxModule::Authorization.verb('archive', 'Archive', [:archive, :restore])])

  def before_filter(filter)
    if filter[:id] == 'archived'
      filter[:strict_match] = true
    end
    filter
  end

  def before_set_targets
    if params[:filtered] && params[:filtered].any? {|a| a[:id] == "archived"}
      @models
    else
      @models.where(archived: false)
    end
  end

  def create
    @model = model_type.new(model_params)
    if @model.save
      @model.broadcast_machine_data(@model, async: true)
      render json: @model, status: :created, location: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def update
    if @model.update(model_params)
      @model.broadcast_machine_data(@model, async: true)
      render json: @model, status: :ok, location: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def destroy
    Rails::logger.warn "Cannot delete a machine from Rx-Data"
    render json: {error: "Cannot delete a machine from Rx-Data"}, status: :unprocessable_entity
  end

  def archive
    if @model.update(archived: true)
      # Remove machine from machine groups
      @model.machine_groups.clear
      render json: @model, status: :ok, location: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def restore
    if @model.update(archived: false)
      render json: @model, status: :ok, location: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def job_data
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      @start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      @end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @machine = Machine.not_archived.find_by(sn: params[:sn])
      @job_data = if @machine
                    JobDatum
                      .joins(job: :job_processes)
                      .includes(:job)
                      .where(job: {job_processes: {machine: @machine}}, created_at: @start_time..@end_time)
                      .limit(@limit).offset(@offset)
                  else
                    JobDatum.none
                  end
      render :job_data
    end
  end

  def jobs
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      @start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      @end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @machine = Machine.not_archived.find_by(sn: params[:sn])
      @jobs = if @machine
                Job.joins(:job_states).where(job_states: {machine: @machine, time: @start_time..@end_time}).limit(@limit).offset(@offset)
              else
                Job.none
              end
      render :jobs
    end
  end

  def job_process_parameters
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      @start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      @end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @machine = Machine.not_archived.find_by(sn: params[:sn])
      @job_process_parameters = if @machine
                                  JobProcessParameter.includes(:job_process).where(job_processes: {machine: @machine, time: @start_time..@end_time}).limit(@limit).offset(@offset)
                                else
                                  JobProcessParameter.none
                                end
      render :job_process_parameters
    end
  end

  def job_states
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      @start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      @end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @machine = Machine.not_archived.find_by(sn: params[:sn])
      if @machine
        @job_states = JobState.includes(:job).where(machine: @machine, time: @start_time..@end_time).limit(@limit).offset(@offset)
        if params.has_key? :desc_sort
          @job_states = @job_states.order(created_at: :desc)
        end
        render :job_states
      else
        render json: {error: "Machine does not exist"}, status: :not_found
      end
    end
  end

  def process_info
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      @start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      @end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @machine = Machine.not_archived.find_by(sn: params[:sn])
      if @machine
        @job_states = JobState.where(machine: @machine, time: @start_time..@end_time)
        render :process_info
      else
        render json: {error: "Machine does not exist"}, status: :not_found
      end
    end
  end

  def status_history
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      @limit = params[:limit] || 200
      @offset = params[:offset] || 0
      @start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      @end_time = Time.zone.parse(params[:endTime] || "")  || Time.current

      machines = Machine.all
      machines = machines.where(sn: params[:resourceSn]) if params[:resourceSn]
      # TODO: machines.where(resource_type_name: params[:resourceTypeName]) if params[:resourceTypeName]
      if params[:resourceGroupName]
        group = MachineGroup.find_by(name: params[:resourceGroupName])
        machines = machines.where(machine_group: group)
      end
      @machine_states = MachineState.includes(:machine).where(machine: machines, time: @start_time..@end_time).order(:time).limit(@limit).offset(@offset)
      @data = @machine_states.group_by(&:machine)

      # insert the previous state to the front of the list
      if @offset == 0
        @data.each do |machine, machine_states|
          @data[machine] = MachineState.where(machine: machine, time: Time.at(0)..@start_time).order("time DESC").limit(1) + machine_states
        end
      end
      render :status_history
    end
  end

  #XBA: probably need to remove this since its copy of status_all? (August 2022)
  def status_last
    @machines = Machine.not_archived.all
    render :status_last
  end

  def status_all
    render json: Machine.status_all
  end

  def machine_last_state
    render json: @model.last_state.to_json, status: :ok
  end

  def last_job_states_per_machine
    render json: Machine.last_job_states(limit_param)
  end

  def production_data
    report_end = Time.zone.now.end_of_hour
    report_start = (report_end - 7.hours).beginning_of_hour
    machine_ids = [@model.id]
    resp = @model.build_production_view_report_data(report_start, report_end, machine_ids)
    render json: resp.to_json, status: :ok
  end

  private

  # Never trust parameters from the scary internet, only allow the white list through.
  def model_params
    params.require(:machine).permit(:sn, :description, :name, :machine_type_id, :is_conveyor_machine, :push_target_enabled, machine_group_ids: [], machine_ids: [])
  end

  def limit_param
    params.permit(:limit)[:limit]
  end
end
