class PercentUptimeDashboard < RxModule::Dashboards::DualAxisChart
  include EerDashboard
  include MachineConcern
  include MachineDashboard

  #TODO: Disabled until we review EER (June 8th 2022)

=begin
  expose('percent_uptime', 'Machine Uptime', 'View the percentage of time spent in each EER state for the selected machine(s)')
  targets("Machine", "The machine(s) to track", -> { machine_targets })
  parameter(:history_length, "History", "How many much time to go back when fetching data", :timelength, default: 86400, min: 0)
  parameter(:bucket_size, "Interval", "Sample the data with this interval", :timelength, default: 1800, min: 0)
  parameter(:multimode, "Machine Groups", "How to handle multiple machine selections", :string, default: 'average', value_list: [
    { label: 'Show Each Machine', value: "individual" },
    { label: 'Show Average', value: "average" },
  ])
  ui_hint(category: 'EER')
  ui_hint(ytype: 'percentage')
  ui_hint(xtype: 'datetime')
  yunit('%')
=end

  def combine_mode
    :average
  end

  def restrict_to_overtime_eer
    true
  end

  def build_results!
    setup_series!
    generate_downtime_counts!
    add_zero_buckets!
    convert_downtime_count_to_uptime_percent!
  end

  def combine_groups?
    @parameters[:multimode] == 'average'
  end

  def unroll_target?(target)
    target.is_a?(MachineGroup) && !combine_groups?
  end

  def setup_series!
    parsed_targets.each do |target|
      if unroll_target?(target)
        target.machines.each do |sub_target|
          push_series(target_to_id(sub_target), target_to_name(sub_target)) unless get_series(target_to_id(sub_target))
        end
      else
        push_series(target_to_id(target), target_to_name(target)) unless get_series(target_to_id(target))
      end
    end
  end

  def generate_downtime_counts!
    eer_for_range.each do |event|
      this_range = (event.start_time)..(event.start_time + (event.max_time || 0).seconds + (event.time_over_max || 0).seconds)
      buckets.each do |bucket|
        bucket[:active_ranges].each do |active_range|
          intersection_length = intersection_length(this_range, active_range)
          if intersection_length > 0
            pushed_targets = [] # a machine could be included 3 ways: directly requested, included by a machine group, included to calculate an arbitrary average
            parsed_targets.each do |target|
              if event_applies?(target, event)
                if unroll_target?(target)
                  sub_target = target.machines.find { |m| m.id == event.id }
                  unless pushed_targets.include?(sub_target)
                    pushed_targets << sub_target
                    increment_point(target_to_id(sub_target), bucket[:end_time], intersection_length)
                  end
                else
                  unless pushed_targets.include?(target)
                    pushed_targets << target
                    increment_point(target_to_id(target), bucket[:end_time], intersection_length)
                  end
                end
              end
            end
          end
        end
      end
    end
  end

  def add_zero_buckets!
    # anything without an eer event had 0 downtime
    buckets.each do |bucket|
      dataset.each do |d|
        increment_point(d[:id], bucket[:end_time], 0)
      end
    end
  end

  def convert_downtime_count_to_uptime_percent!
    # up to this point we've calculated how much downtime targets have had in seconds, but we want to chart %uptime
    dataset.each do |s|
      target = target_from_id(s[:id])
      time_multiplier = (combine_groups? && target.is_a?(MachineGroup)) ? target.machines.length : 1 # the total available work time for a machine is the sum of all of the available work time for each individual machine
      s[:data].each do |point|
        bucket = buckets.find { |b| b[:end_time] == point[:x] }
        active_time = bucket[:active_time] * time_multiplier
        if active_time == 0
          point[:y] = 0
        else
          point[:y] = (((active_time - point[:y]) / active_time) * 100)
        end
        point[:y] = [[point[:y], 100].min, 0].max # hard-cap for rounding issues
      end
    end
  end

  def buckets
    return @_buckets if @_buckets
    @_buckets = []
    this_time = start_time
    while this_time < end_time
      bucket_range = this_time..([this_time + @parameters[:bucket_size].seconds, end_time].min)
      active_ranges = scheduled_range_data.map do |scheduled_range|
        intersect_range(scheduled_range, bucket_range)
      end.compact
      active_time = active_ranges.reduce(0) do |sum, r|
        sum + range_length(r)
      end.to_f
      buckets.push({
                     end_time: bucket_range.end.utc.iso8601,
                     active_ranges: active_ranges,
                     active_time: active_time
                   })
      this_time = bucket_range.end
    end
    @_buckets
  end

  def start_time
    @_start_time ||= (call_time - (@parameters[:history_length] || 86400).seconds)
  end

  def end_time
    @_end_time ||= call_time
  end

  def raw_data_only_targets
    super - (combine_groups? ? [] : requested_machine_groups.reduce([]) do |arr, mg|
      arr.concat(mg.machines.map do |m|
        target_to_id(m)
      end)
    end)
  end

  class << self
    def machine_targets
      MachineGroup.all.map do |g|
        { label: target_to_name(g), value: target_to_id(g), group: "Machine Group", description: "Report on percent uptime for the entire #{g.name} group" }
      end + Machine.all.map do |m|
        { label: target_to_name(m), value: target_to_id(m), group: "Machine", description: "Report on percent uptime for machine #{m.name}, sn #{m.sn}" }
      end
    end
  end
end