import React from "react";
import {observer} from "mobx-react-lite";
import {FormProps, ModelInput} from "@rx-suite/ui";
import {
    Button,
    Dialog,
    DialogActions,
    DialogContent,
    DialogTitle
} from "@material-ui/core";
import {t, Trans} from "@lingui/macro";
import {ModelProvider} from "@rx-suite/ui";
import {StatusColorModel} from "../../../stores/StatusColorStore";

interface StatusColorEditDialogProps extends FormProps{
    open: boolean,
    handleSubmit: (model:StatusColorModel) => void,
    handleClose: () => void
}

export const StatusColorEditDialog:  React.FunctionComponent<StatusColorEditDialogProps> = observer(({model, open, handleSubmit, handleClose}) => {
    return (
        <Dialog open={open} onClose={handleClose} aria-labelledby="form-dialog-title" fullWidth={true} maxWidth={'md'}>
            <ModelProvider value={model}>
                <DialogTitle id="form-dialog-title">{t`Edit Status Color`}</DialogTitle>
                <DialogContent>
                        <ModelInput label={t`Machine State`} name={"status_name"} disabled/>
                        <ModelInput name={"color"} type={"color"} />
                    <DialogActions>
                        <Button onClick={handleClose} color={"secondary"}><Trans>Close</Trans></Button>
                        <Button disabled={!model.viewModel.isValid} onClick={e => handleSubmit(model)} color={"secondary"}><Trans>Submit</Trans></Button>
                    </DialogActions>
                </DialogContent>
            </ModelProvider>
        </Dialog>
    )
})
