import * as React from 'react';
import ReactTable from "react-table";
import {t} from "@lingui/macro";
import {SidesPaperDisplay} from "../generic/SidesPaperDisplay";
import {ProcessParams} from "../../pages/job_history/JobHistoryOperatorForm";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";

type JobProcessDataTableProps = {
    processData: Readonly<{
        left: ReadonlyArray<ProcessParams>;
        right: ReadonlyArray<ProcessParams>;
    }>;
};


export const JobProcessDataTable: React.FunctionComponent<JobProcessDataTableProps> = ({ processData }) => {

    // displays the name and the value of the process params in a table

    const leftTableData: Array<ProcessParams> = processData.left.slice();

    const rightTableData: Array<ProcessParams> = processData.right.slice();

    const columns = [{
        Header: t`Tag`,
        accessor: 'name',
        filterable: true
    }, {
        Header: t`Value`,
        accessor: 'value'
    }];

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    return <SidesPaperDisplay
        leftDisplay={{
            label: t`Left`,
            component: <ReactTable
                data={leftTableData}
                columns={columns}
                showPagination={false}
                // provide all of the data to the table,
                // without this, the table could show 30 rows,
                // but only fill 20 rows with data
                defaultPageSize={leftTableData.length}
                minRows={leftTableData.length <= 0 ? 10 : leftTableData.length} // if empty, pad the table with 10 rows
                sortable={false}
            />
        }}
        rightDisplay={{
            label: t`Right`,
            component: <ReactTable
                data={rightTableData}
                columns={columns}
                showPagination={false}
                // provide all of the data to the table,
                // without this, the table could show 30 rows,
                // but only fill 20 rows with data
                defaultPageSize={rightTableData.length}
                minRows={rightTableData.length <= 0 ? 10 : rightTableData.length} // if empty, pad the table with 10 rows
                sortable={false}
            />
        }}
        invertSides={displayRightLensFirst}
    />;
};
