import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {MachineDataViewTab} from "../MachineDataViewTab";
import {JobStateModel, JobStateStore} from "../../../stores/JobStateStore";
import {JobModel} from "../../../stores/JobStore";
import {MachineDataViewPanelProps} from "../MachineDataView";
import {ArrowDropDown, ArrowRight} from "@material-ui/icons";
import {useJobSideColumn} from "../../../hooks/table_columns/useJobSideColumn";
import {AsyncProcessParams} from "../sub_components/AsyncProcessParams";
import {makeFormattedDateTimeColumn} from "@rx-suite/ui";

export const JobsInfoPanel: React.FunctionComponent<MachineDataViewPanelProps> = observer((props) => {
    const {machine} = props;
    const store = JobStateStore;

    const columns = [
        {
            Header: t`Jobs`,
            columns: [
                {
                    expander: true,
                    Header: () => null,
                    Expander: ({ isExpanded, ...rest }) => {
                        if(rest.original.state?.toLowerCase() === 'finished') {
                            return(
                                <div>
                                    {isExpanded ? <ArrowDropDown/> : <ArrowRight/>}
                                </div>
                            )
                        } else {
                            return null
                        }
                    },
                },
                {
                    Header: t`Job Number`,
                    id: 'job_number',
                    filterable: true,
                    accessor: (jobState: JobStateModel) => {
                        const job = jobState.job as JobModel;

                        if(!job){
                            return 'n/a';
                        }
                        return job.number;
                    }
                },
                useJobSideColumn(machine, `side`),
                makeFormattedDateTimeColumn("time", t`Date - Time`, "dateTime")
            ]
        }
    ];

    const processParamsSubComponent = (original: JobStateModel) => {
        if (original.state != 'finished') {
            return null;
        }
        return <AsyncProcessParams model={original} />;
    }

    return (
        <MachineDataViewTab
            columns={columns}
            machine={machine}
            enabledKey={'job_state_enabled'}
            tabName={'Job'}
            store={store}
            defaultFiltered={[{id: 'state', value: 5}]}
            subComponent={processParamsSubComponent}
            exportedFileName={'Jobs.csv'}
            exportEndpoint={'jobs'}
        />
    );
});
