import * as React from "react";
import {observer} from "mobx-react-lite";
import {BarChart, ReportComponentMetaData, TemplateReportComponent} from "@rx-suite/ui";
import {useCallback, useState} from "react";
import {useMachineErrorToolTip} from "../../../hooks/reports/machine_error/useMachineErrorToolTip";

type MachineAbortReportProps = {
    component: {
        meta: ReportComponentMetaData;
        data: any;
        extra_data: any;
    };
};

export const MachineAbortReport: React.FunctionComponent<MachineAbortReportProps> = observer(({component}) => {
    const data = component.data;
    const [hasPercent, setHasPercent] = useState(false);

    const buildToolTip = useCallback(({series, seriesIndex, dataPointIndex, w}) => {
        if(!component.extra_data) { return (series[seriesIndex][dataPointIndex].toFixed(0)) + "%"; }
        setHasPercent(true);

        const color = w.config.colors[seriesIndex]

        return useMachineErrorToolTip(component.extra_data[seriesIndex][dataPointIndex], color)
    }, [component.extra_data])

    const labelFormatter = useCallback((value, index) => {
        if(!component.extra_data) { return (value.toFixed(0)) + "%"; }
        setHasPercent(true);

        const info = component.extra_data[index.seriesIndex][index.dataPointIndex];
        return `${info.percent}% (${info.count})`
    }, [component.extra_data])

    return (
        <TemplateReportComponent meta={component.meta}>
            <BarChart
                series={data.series}
                options={
                    {
                        title: {
                            text: 'Top Machine Aborts',
                            align: 'center'
                        },
                        chart: {
                            toolbar: {
                                show: false
                            }
                        },
                        legend: {
                            show: false
                        },
                        dataLabels: {
                            formatter: labelFormatter
                        },
                        tooltip: {
                            custom: buildToolTip
                        },
                        plotOptions: {
                            bar: {
                                barHeight: '60%'
                            }
                        },
                        xaxis: {
                            categories: data.categories,
                            labels: {
                                formatter: (value: number) => {
                                    const suffix = hasPercent? "%" : '';
                                    return (value.toFixed(0)) + suffix
                                }
                            },
                            tickAmount: 6,
                        },
                        yaxis: {
                            labels: {
                                maxWidth: 400,
                            },
                            // If less than 100, round up to nearest 10s
                            max: (max: number) => max < 100 ? Math.ceil(max / 20) * 20 : 100,
                            tickAmount: 6,
                            forceNiceScale: true
                        },
                        colors: data.colors ? data.colors : ['#FF0000']
                    }
                }
                height={'100%'}
                horizontal/>
        </TemplateReportComponent>
    );
})