import * as React from 'react';
import {EventData, EventModel} from "../../../stores/EventStore";
import {useEffect, useState} from "react";
import {useSnackbar} from "notistack";
import {CircularProgress, makeStyles, Typography} from "@material-ui/core";
import {EventDataTable} from "./EventDataTable";
import {t} from "@lingui/macro";

type AsyncEventDataProps = {
    model: EventModel;
};

const useStyles = makeStyles((theme) => ({
    eventDataTable: {
        "& .rt-tbody": {
            height: 'unset'
        }
    },
    loadingContainer: {
        display: 'flex',
        flexDirection: 'row',
        alignItems: 'center',
        justifyContent: 'center',
        padding: theme.spacing(2)
    },
    loadingText: {
        marginLeft: theme.spacing(1)
    }
}));

export const AsyncEventData: React.FunctionComponent<AsyncEventDataProps> = ({ model }) => {

    const classes = useStyles();

    const [eventData, setEventData] = useState<EventData|null>(null);

    const {enqueueSnackbar} = useSnackbar();

    useEffect(() => {
        const fetchData = async () => {
            const eventDataForEvent = await model.getEventData();

            if ('error' in eventDataForEvent) {
                enqueueSnackbar(eventDataForEvent.error, {variant: "error"});
                setEventData([]);
                return;
            }

            setEventData(eventDataForEvent.event_data);
        };
        fetchData();
    }, []);

    if (!eventData) {
        return <div className={classes.loadingContainer}>
            <CircularProgress />
            <Typography className={classes.loadingText} variant="subtitle1">
                {t`Fetching Data…`}
            </Typography>
        </div>;
    }

    return <div className={classes.eventDataTable}>
        <EventDataTable eventData={eventData} />
    </div>;
};
