import * as React from "react";
import {renderToString} from "react-dom/server";
import {Trans} from "@lingui/macro";
import Box from "@material-ui/core/Box";
import {Divider, Grid, makeStyles, Typography} from "@material-ui/core";
import {observer} from "mobx-react-lite";
import classNames from "classnames";
import {Conditional, IfTrue} from "@rx-suite/ui";

export type MachineErrorDatum = {
    name: string;
    data: {count: number, percent: number}[];
}

type MachineErrorToolTip = {
    extraData: {name: string, count: number, percent: number},
    color: string;
}

const useStyles = makeStyles((theme) => ({
    title: {
        fontWeight: 'bold',
        fontSize: '0.9rem',
        margin: '2px'
    },
    text: {
        fontSize: '0.85rem'
    },
    statusWrapper: {
        width: "fit-content",
        fontSize: '0.8rem'
    },
    listLabel: {
        fontWeight: 'bold',
        marginRight: theme.spacing(1)
    },
}));

const MachineErrorToolTip: React.FunctionComponent<MachineErrorToolTip> = observer(({extraData, color}) => {
    const classes = useStyles();
    const hasPercent = !!extraData.percent;

    if(!extraData){
        return (
            <Box mb={1} mt={1} ml={1} mr={1}>
                <Trans>No Data</Trans>
            </Box>
        )
    }

    return (
        <Box mb={1} mt={1} ml={1} mr={1}>
            <Typography className={classes.title} style={{backgroundColor: color}}>{extraData.name}</Typography>
            <Divider/>

            <Grid container justifyContent={'space-between'}>
                <Typography className={classNames(classes.listLabel, classes.text)}>Count:</Typography>
                <Typography className={classes.text}>{extraData.count}</Typography>
            </Grid>
            <Conditional condition={hasPercent}>
                <IfTrue>
                    <Grid container justifyContent={'space-between'}>
                        <Typography className={classNames(classes.listLabel, classes.text)}>Percent:</Typography>
                        <Typography className={classes.text}>{extraData.percent}%</Typography>
                    </Grid>
                </IfTrue>
            </Conditional>
        </Box>
    )
});

export const useMachineErrorToolTip = (extraData: {name: string, count: number, percent: number}, color: string) => {
    // Apex Chart custom tool tip cannot take direct React components
    return renderToString(
        <MachineErrorToolTip extraData={extraData} color={color}/>
    )
}
