import * as React from "react";
import { t } from "@lingui/macro"
import {MachineModel, MachineStore} from "../../stores/MachineStore";
import {Grid} from "@material-ui/core";
import {BreadModelListBadge} from "../../components/generic/bread_model_list_badges/BreadModelListBadge";
import {Model} from "@rx-suite/data";

type ModelWithMachineAssociation = {
    machine_ids: string[]
} & Model

export const useMachineListColumn = (useEmptyListAsAll=false) => {
    const machines = MachineStore.getAll() as MachineModel[];

    return (
        {
            Header: t`Machines`,
            accessor: (m: MachineModel) => m,
            id: 'machine_ids',
            Cell: (cellInfo: {value: ModelWithMachineAssociation }) => {
                const model = cellInfo.value as ModelWithMachineAssociation;

                if(useEmptyListAsAll && model.machine_ids.length == 0){
                    return (
                        <Grid container>
                            <BreadModelListBadge text={t`All Machines`}/>
                        </Grid>

                    )
                }

                return (
                    <Grid container>
                        {model.machine_ids.map((machine_id) => {
                            const machine = machines.find(m=> m.id === machine_id);

                            if(!machine) {
                                return null;
                            }

                            return <BreadModelListBadge text={machine.extendedName}/>
                        })}
                    </Grid>
                )
            }
        }
    )
}

