import * as React from 'react';
import {BreadModel, makeColumn, SelectField} from "@rx-suite/ui";
import {BlockPickingModel, BlockPickingStore} from "../../stores/BlockPickingStore";
import {BlockPickingForm} from "./BlockPickingForm";
import {t} from "@lingui/macro";
import {DcsConnectionModel, DcsConnectionStore} from "@rx-suite/stores";
import {LinearProgress} from "@material-ui/core";

export const BlockPickingBreadModel = () => {
    const dcsModels = DcsConnectionStore.getAll() as DcsConnectionModel[];

    const columns = React.useMemo(() => {
        return [
            makeColumn('name', t`Workstation Name`, "string"),
            {
                Header: t`LMS Connection`,
                id: 'lms_connection',
                accessor: (workstation :BlockPickingModel) => workstation,
                filterable: false,
                Cell: (cellInfo: {value: BlockPickingModel }) => {
                    if(!cellInfo.value.external_dcs_id){
                        return null;
                    }

                    const model = dcsModels.find(connection => connection.id === cellInfo.value.external_dcs_id);

                    if(!model){
                        return t`Unknown`;
                    }

                    return (model.resolved_name)
                }
            },
        ];
    }, [dcsModels]);

    return <BreadModel store={BlockPickingStore} Form={BlockPickingForm} columns={columns}/>;
};