import * as React from 'react';
import {BreadModel, makeColumn, ModelTableDeleteButton, ModelTableUpdateButton} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {AbortFilterStore} from "../../../stores/AbortFilterStore";
import {AbortFilterForm} from "./AbortFilterForm";
import {observer} from "mobx-react-lite";
import {ArchiveButton} from "../../../components/machine/settings_table/ArchiveButton";
import {RestoreButton} from "../../../components/machine/settings_table/RestoreButton";
import {ToggleArchivedButton} from "../../../components/machine/settings_table/ToggleArchivedButton";

export const AbortFilterBreadModel = observer(() => {
    const columns = React.useMemo( () => {
        return [
            makeColumn('name', t`Rule Name`, "string"),
            makeColumn('regex', t`Regex Filters`, "string"),
            makeColumn('replace_name', t`Rename To`, "string")
        ]
    }, [])

    return (
        <BreadModel
            store={AbortFilterStore}
            Form={AbortFilterForm}
            columns={columns}
            tableProps={
                {
                    replacementActions: {
                        Edit: (cell) => (cell.model.name === "Other Error") ? null
                            : <ModelTableUpdateButton store={cell.store} model={cell.model}/>,
                        Delete: (cell) => cell.model.archived
                            ? <RestoreButton model={cell.model}/>
                            : <ArchiveButton model={cell.model}/>
                    },
                    preTableActions: <ToggleArchivedButton store={AbortFilterStore}/>,
                    suppressWrapper: true
                }
            }
        />
    )
});