import * as React from 'react';
import {BreadModel, makeColumn, ModelTableDeleteButton, ModelTableUpdateButton, SelectField} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {ErrorFilterStore} from "../../../stores/ErrorFilterStore";
import {ErrorFilterForm} from "./ErrorFilterForm";
import {observer, Observer} from "mobx-react-lite";
import {ToggleArchivedButton} from "../../../components/machine/settings_table/ToggleArchivedButton";
import {RestoreButton} from "../../../components/machine/settings_table/RestoreButton";
import {ArchiveButton} from "../../../components/machine/settings_table/ArchiveButton";
import {Model} from "@rx-suite/data";

export const ErrorFilterBreadModel = observer(() => {
    const BehaviourString = {
        'ignore': 'Ignore',
        'replace': 'Replace',
        'use_as_is': 'Use As Is'
    }
    const columns = React.useMemo(() => {
        return [
            makeColumn('name', t`Rule Name`, "string"),
            makeColumn('error_codes', t`Error Codes`, "string"),
            {
                Header: t`Behaviour`,
                id: 'behaviour',
                accessor: (model: Model) => model,
                Cell: (row: { original: Model | null | undefined }) => {
                    return (
                        <Observer>
                            {
                                () => {
                                    let value = null;
                                    if (row.original) {
                                        value = BehaviourString[row.original.dig('behaviour')];
                                    }
                                    return (
                                        <span title={value}>
                                    {value}
                                </span>
                                    )
                                }
                            }
                        </Observer>
                    )
                },
                filterable: true,
                sortable: true,
                Filter: (params: { filter?: { value: string } | undefined, onChange: (newVal: string | boolean) => void }) => {
                    return <Observer>
                        {
                            () => {
                                const behaviours = [
                                    {label: t`Ignore`, value: 0},
                                    {label: t`Replace`, value: 1},
                                    {label: t`Use As Is`, value: 2},
                                ]
                                return <SelectField value={params.filter?.value || ''}
                                                    options={behaviours}
                                                    onChange={(name, val) => {
                                                        params.onChange(val);
                                                    }}
                                                    clearable
                                />
                            }
                        }
                    </Observer>
                }
            }
        ]
    }, [ErrorFilterStore]);

    return (
        <BreadModel
            store={ErrorFilterStore}
            Form={ErrorFilterForm}
            columns={columns}
            tableProps={
                {
                    replacementActions: {
                        Edit: (cell) => (cell.model.name === "Other Error") ? null
                            : <ModelTableUpdateButton store={cell.store} model={cell.model} />,
                        Delete: (cell) => cell.model.archived
                            ? <RestoreButton model={cell.model}/>
                            : <ArchiveButton model={cell.model}/>
                    },
                    preTableActions: <ToggleArchivedButton store={ErrorFilterStore}/>,
                    suppressWrapper: true
                }
            }
        />
    )
});