import * as React from 'react';
import {BreadModel, makeColumn} from "@rx-suite/ui";
import {MachineDataPushTargetModel, MachineDataPushTargetStore} from "../../../stores/MachineDataPushTargetStore";
import {ForwardingTargetsExternalForm} from "./ForwardingTargetsExternalForm";
import {t} from "@lingui/macro";
import {Grid} from "@material-ui/core";
import {
    BreadModelListBadgeEnabled
} from "../../../components/generic/bread_model_list_badges/BreadModelListBadgeEnabled";
import {useMachineListColumn} from "../../../hooks/table_columns/useMachineListColumn";
import {observer} from "mobx-react-lite";

export const ForwardingTargetsExternalBreadModel = observer(() => {
    const buildEndpoints = React.useCallback((model: MachineDataPushTargetModel) => {
        const endpoints = [
            {key: 'order_path', label: 'Orders'},
            {key: 'machine_state_path', label: 'Machine States'},
            {key: 'job_state_path', label: 'Job States'},
            {key: 'error_path', label: 'Errors'},
            {key: 'event_path', label: 'Events'},
            {key: 'maintenance_path', label: 'Maintenances'},
            {key: 'process_data_path', label: 'Process Parameters'}
        ];

        return (
            <Grid container direction={'column'}>
                {
                    endpoints.map(endpoint => {
                        if(model[endpoint.key] !== null && model[endpoint.key] !== '') {
                            return <BreadModelListBadgeEnabled text={endpoint.label}/>
                        }else{
                            return null;
                        }
                    })
                }
            </Grid>
        );
    }, []);

    const columns = React.useMemo(() => {
        return [
            makeColumn('name', t`System Name`, "string"),
            {
                Header: t`Endpoints`,
                accessor: (targets: MachineDataPushTargetModel) => targets,
                id: 'endpoints',
                Cell: (cellInfo: {value: MachineDataPushTargetModel}) => {
                    return buildEndpoints(cellInfo.value);
                }
            },
            useMachineListColumn(true)
        ]
    }, []);


    return(
        <BreadModel
            store={MachineDataPushTargetStore}
            Form={ForwardingTargetsExternalForm}
            columns={columns}
        />
    )
});
