import * as React from 'react';
import {MachineStore} from "../../../stores/MachineStore";
import {BreadModel, makeColumn} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {observer} from "mobx-react-lite";
import {MachineWorkstation} from "./MachineWorkstation";
import {MachineGroupStore} from "../../../stores/MachineGroupStore";
import {MachineTypeStore} from "../../../stores/MachineTypeStore";
import {useMachineGroupColumn} from "../../../hooks/table_columns/useMachineGroupColumn";
import {useMachineTypeColumn} from "../../../hooks/table_columns/useMachineTypeColumn";

export const MachinesWorkstationTable = observer(() => {
    const columns = React.useMemo(() => {
        return [
            makeColumn('name', t`Name`, "string"),
            makeColumn('sn', t`Serial Number`, "string"),
            useMachineTypeColumn(),
            useMachineGroupColumn(),
        ]
    }, [MachineGroupStore.models, MachineTypeStore.models]);

    return (
        <BreadModel
            store={MachineStore}
            Form={MachineWorkstation}
            columns={columns}
            mountCreate={false}
            mountEdit={false}
            preferReadLink
            tableProps={
                {
                    replacementActions: {
                        Delete: () => null,
                        Edit: () => null
                    },
                    defaultFiltered: [{ id: 'archived', value: false }],
                    preferReadLink: true,
                    showCreateButton: false,
                    suppressActionColumn: true
                }
            }
        />
    )
});
