import {attribute, DataStore, model, Model} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from "yup";

@model({
  transportLayer: "/api/data_archive_settings",
  permissionTarget: 'DataArchiveSetting',
  routingInfo: {
    create: "/data_archive_settings/create",
    update: "/data_archive_settings/:id/edit",
    read: "/data_archive_settings/:id"
  },
  singleton: true
})
export class DataArchiveSettingModel extends Model {
  @attribute({ label: t`Max Job Age (Hours)`, validation: yup.number().min(0).max(8760) })
  job_max_age: number = 336;

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  @attribute({ label: t`Delete Count Order`, validation: yup.number().min(0) })
  delete_count_order: number = 3;

  @attribute({ label: t`Delete Count Job`, validation: yup.number().min(0) })
  delete_count_job: number = 5;

  @attribute({ label: t`Delete Count Machine State`, validation: yup.number().min(0) })
  delete_count_machine_state: number = 4;

  @attribute({ label: t`Delete Count Error`, validation: yup.number().min(0) })
  delete_count_error: number = 3;

  @attribute({ label: t`Delete Count Maintenance`, validation: yup.number().min(0) })
  delete_count_maintenance: number = 2;

  @attribute({ label: t`Delete Count Event`, validation: yup.number().min(0) })
  delete_count_event: number = 2;

  @attribute({ label: t`Delete Count Calibration`, validation: yup.number().min(0) })
  delete_count_calibration: number = 2;

  @attribute({ label: t`Delete Count KPI Cache`, validation: yup.number().min(0) })
  delete_count_kpi_cache: number = 4;

  @attribute({ label: t`Cycle Interval (Seconds)`, validation: yup.number().min(1) })
  cycle_interval: number = 10;

  constructor() {
    super();
    super.setup();
  }
}

export const DataArchiveSettingStore = DataArchiveSettingModel.store as DataStore<DataArchiveSettingModel>;
