import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro";
import * as yup from "yup";
import {override} from "mobx";
import {JobProcessStore} from "./JobProcessStore";

@model({
  transportLayer: "/api/job_process_parameters",
  routingInfo: {}
})

export class JobProcessParameterModel extends Model {
  @attribute({label: t`Job Process`, connects: "JobProcessModel"})
  job_process_id: string | null = null;

  @attribute({ label: t`Name`, validation: yup.string().nullable()})
  name: string | null = null;

  @attribute({ label: t`Value`, validation: yup.string().nullable()})
  value: string | null = null;

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  @override
  updateFromJson(json: Model, isClone?: boolean) {
    super.updateFromJson(json, isClone);
    let job_process = JobProcessStore.data.get(json.job_process.id.toString());
    if(!job_process){
      job_process = JobProcessStore.createModel(json.job_process);
      JobProcessStore.setModel(job_process)
    }
    this.job_process_id = job_process.getId()
  }

  constructor() {
    super();
    super.setup();
  }
}

export const JobProcessParameterStore = JobProcessParameterModel.store as DataStore<JobProcessParameterModel>;