import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro"
import * as yup from "yup";
import {override} from "mobx";
import {JobStore} from "./JobStore";

@model({
  transportLayer: "/api/job_processes",
  routingInfo: {}
})

export class JobProcessModel extends Model {
    @attribute({label: t`Job`, connects: "JobModel"})
    job_id: string | null = null;

    @attribute({label: t`Job`, connects: "Machine"})
    machine_id: string | null = null;

    @attribute({ label: t`Time`, validation: yup.date().required() })
    time: Date = new Date();

    @attribute({ label: t`Side`, validation: yup.string().required() })
    side: string  = "";

    @attribute({ label: t`Created At`, autoForm: false })
    created_at: Date = new Date();

    @attribute({ label: t`Updated At`, autoForm: false })
    updated_at: Date = new Date();

    static types = {
        time: "date"
    }

    @override
    updateFromJson(json: Model, isClone?: boolean) {
        super.updateFromJson(json, isClone);
        let job = JobStore.data.get(json.job.id.toString());
        if(!job){
            job = JobStore.createModel(json.job);
            JobStore.setModel(job)
        }
        this.job_id = job.getId()
    }

    constructor() {
        super();
        super.setup();
    }
}

export const JobProcessStore = JobProcessModel.store as DataStore<JobProcessModel>;