import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {action} from "mobx";

@model({
    transportLayer: "/api/process_booking_workstations",
    permissionTarget: "ProcessBookingWorkstation",
    routingInfo: {
        create: '/settings/process_booking_workstations/create',
        read: '/settings/process_booking_workstations/:id/edit',
        update: '/settings/process_booking_workstations/:id/edit'
    }
})
export class ProcessBookingWorkstationModel extends Model {
    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Name is required`)
    })
    name: string = '';
    
    @attribute({
        label: t`Machines`,
        validation: yup.array()
    })
    machine_ids: number[] = [];

    @action.bound
    async bookProcess(machineSn: string, jobNumber: string) {
        const resp = await fetch(`${ProcessBookingWorkstationStore.transportLayer.base}/${this.id}/book`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                job_number: jobNumber,
                machine_sn: machineSn
            })
        });

        const data = await resp.json();

        if(data['error']){
            console.error(data['details']);
            throw data['error'];
        }
    }

    constructor() {
        super();
        super.setup();
    }
}

export const ProcessBookingWorkstationStore = ProcessBookingWorkstationModel.store as DataStore<ProcessBookingWorkstationModel>