class ForwardMachineDataJob < ApplicationJob
  queue_as :machine_data_forwarding

  def perform(machine_id, forwarding_path, params, data)
    machine = Machine.find_by(id: machine_id)

    return if machine.nil?

    push_targets = machine.push_targets_for_path(forwarding_path)

    push_targets.each do |target|
      endpoint = target[forwarding_path]

      next if endpoint.blank?

      url = "#{target.base_url}/#{endpoint}"
      Rails.logger.info "Forwarding to #{url}. Data => #{data}"
      response = Faraday.send(:put, url) do |req|
        req.params['sn'] = params[:sn]
        req.params['date'] = params[:date]
        req.params['time'] = params[:time]
        req.headers['Content-Type'] = 'application/json'
        req.body = data.to_json
      end
      parsed_response = begin
                          Oj.load(response.body, symbol_keys: true)
                        rescue # If json parsing fails, pass through the body
                          response.body
                        end
      raise parsed_response[:error] || parsed_response[:err] || parsed_response.to_s || "Unknown error - consult logs" if response.status == 500
      parsed_response
    rescue => err
      Rails.logger.error "Failed to forward to #{url}. Data => : #{data}. Error: #{err}"
    end
  end
end
