module KpiTableHandler
  module Base
    class PgKpiTableDatabaseHandler
      include KpiHelpers
      attr_accessor :use_offset

      def initialize(use_offset)
        self.use_offset = use_offset
      end

      def from_sub_query(table, base_select, base_group)
        table.select(base_select + [select_sub_query]).group(base_group + [group_sub_query])
      end

      def select_sub_query
        if use_offset
          string = <<~SQL
            CAST(
                  CASE
                    WHEN time - date_trunc('hour', time) = INTERVAL '0' minute
                      then time - INTERVAL '30' minute
                    ELSE
                      time
                  END as timestamp
            ) as time
          SQL
          string.squish
        else
          "date_trunc('hour', time) as time"
        end
      end

      def group_sub_query
        if use_offset
          string = <<~SQL
            CAST(
                  CASE
                    WHEN time - date_trunc('hour', time) = INTERVAL '0' minute
                      then time - INTERVAL '30' minute
                    ELSE
                      time
                  END as timestamp
            )
          SQL
          string.squish
        else
          "date_trunc('hour', time)"
        end
      end

      def group_kpi_table_by_time_frame(sub_query, timeframe, timeframe_select, timeframe_group, start_time)
        select_query = timeframe_select
        group_query = timeframe_group
        case timeframe
        when Timeframe::DAY_BY_DAY
          select_query << "date_trunc('day', time::timestamp AT TIME ZONE 'UTC' AT TIME ZONE '#{ENV['TZ']}') as time"
          group_query << "date_trunc('day', time::timestamp AT TIME ZONE 'UTC' AT TIME ZONE '#{ENV['TZ']}')"
        when Timeframe::WEEK_BY_WEEK
          # 1 get number of seconds since epoch
          # 2 divide by number of seconds in a day (86400) to put get number of days since epoch
          # 3 epoch is on a thursday, so we need to subtract the difference since last thursday
          days_since_thursday = ((start_time - (start_time.monday + 3.days)) / 1.day).floor
          # 4 divide by 7 to get number of weeks since epoch
          # 5 then we floor
          # 6 then we do the opposite of operations 2, 3 and 4 in backwards order to get a date time again
          select_query << "to_timestamp((FLOOR((EXTRACT(EPOCH from time::timestamp AT TIME ZONE 'UTC' AT TIME ZONE '#{ENV['TZ']}')" +
            "/ 86400 - #{days_since_thursday}) / 7 ) * 7 + #{days_since_thursday}) * 86400) as time"
          group_query << "to_timestamp((FLOOR((EXTRACT(EPOCH from time::timestamp AT TIME ZONE 'UTC' AT TIME ZONE '#{ENV['TZ']}')" +
            "/ 86400 - #{days_since_thursday}) / 7 ) * 7 + #{days_since_thursday}) * 86400)"

          #days_since_monday = ((start_time - start_time.monday) / 1.day).floor
          #select_query << "date_trunc('week', time::timestamp AT TIME ZONE 'UTC' AT TIME ZONE '#{ENV['TZ']}') + interval '#{days_since_monday} day' as time"
          #group_query << "date_trunc('week', time::timestamp AT TIME ZONE 'UTC' AT TIME ZONE '#{ENV['TZ']}') + interval '#{days_since_monday} day'"
        when Timeframe::ALL
        else
          return sub_query
        end
        from_sub_query = sub_table_to_from_string(sub_query)
        table = sub_query.klass
        table.select(select_query).from(from_sub_query).group(group_query)
      end
    end
  end
end