module KpiTableHandler
  class MachineStateDurationKpiHandler < KpiTableHandler::Base::KpiTableHandler
    include KpiHelpers

    class MachineStateNotSpecified < StandardError; end

    attr_accessor :machine_states, :sum_as

    def initialize(start_time, end_time, timeframe, machine_ids, params: {}, options: {})
      raise MachineStateNotSpecified unless params.key?(:machine_states)

      self.machine_states = params[:machine_states]
      self.sum_as = params[:sum_as]

      super(start_time, end_time, timeframe, machine_ids, options: options)
    end

    def table
      KpiMachineStateDuration
    end

    def base_select
      select = machine_states.map do |state|
        # Have to wrap sum as string with "" because states like "on" and "off" are reserved sql keywords
        table.arel_table[state.to_sym].sum.as("\"#{state.to_s}\"")
      end

      #TODO: probably add as base to kpi handler?
      select << table.arel_table[:machine_id].as("machine_id") unless aggregate_machines?


      select
    end

    def kpi_to_extract
      query = if as_sum?
                "(#{machine_states.join(" + ")}) as \"#{sum_as}\""
              else
                machine_states.map {|m| "\"#{m.to_s}\""}.join(", ")
              end

      #TODO should probably be generic?
      query += ", machine_id" unless aggregate_machines?

      query
    end

    def timeframe_select
      selects = []
      if as_sum?
        selects << "SUM(#{sum_as}) as \"#{sum_as}\""
        selects << table.arel_table[:machine_id] unless aggregate_machines?
      else
        selects.concat(base_select)
      end
      selects
    end

    def timeframe_group
      base_group
    end

    def col_name
      cols = []
      if as_sum?
        cols << sum_as
      else
        cols.concat(machine_states.map {|state| state.to_sym})
      end

      #TODO: should be in base KPI?
      cols << :machine_id unless aggregate_machines?
      cols
    end

    # record is still an active record at this point, so extract the values we want
    def record_value(record)
      result = {}

      if as_sum?
         result[sum_as.to_sym] = record[sum_as.to_sym]
      else
        machine_states.each do |state|
          result[state.to_sym] = record[state.to_sym]
        end
      end
      result
    end

    def as_sum?
      return false if sum_as.nil?
      true
    end

    def hash_targets_col
      machine_states.map{|m| m.to_sym}
    end

    def apply_hash_kpi_operation(hash)
      hash = hash_sum_col(hash)

      hash
    end

    # For this KPI we want to treat the 'now bucket' as a future bucket
    def get_now_intervals(bucket_start_time, bucket_end_time)
      first_now_bucket_time = KpiHelpers.kpi_bucket_time(data_expired_date + 1.days)
      return nil if bucket_end_time < first_now_bucket_time
      last_now_bucket_time = KpiHelpers.kpi_bucket_time(Time.zone.now)
      return nil if bucket_start_time > last_now_bucket_time
      {
        bucket_start_time: [bucket_start_time, first_now_bucket_time].max,
        bucket_end_time: [bucket_end_time, last_now_bucket_time].min
      }
    end

    def get_future_intervals(bucket_start_time, bucket_end_time)
      first_future_bucket_time = KpiHelpers.kpi_bucket_time(Time.zone.now)
      return nil if bucket_end_time < first_future_bucket_time
      {
        bucket_start_time: [bucket_start_time, first_future_bucket_time].max,
        bucket_end_time: bucket_end_time
      }
    end

    private

    # Sum Key values and remap to new key
    def hash_sum_col(hash)
      return hash unless as_sum?

      if aggregate_machines?
        hash = buckets_sum_as_transformation(hash)
      else
        hash.each do |machine_id, buckets|
          hash[machine_id] = buckets_sum_as_transformation(buckets)
        end
      end

      hash
    end

    def buckets_sum_as_transformation(buckets)
      buckets.each do |time_bucket, data|
        buckets[time_bucket] = {sum_as.to_sym => data.values.sum}
      end
      buckets
    end
  end
end