module MachineKpiCalculator
  class BoxCoaterStateBreakdown < MachineKpiCalculator
    include KpiHelpers
    include BucketCalculator

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {})
        kpi = BoxCoaterStateBreakdown.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :time_in_state_breakdown,
          handler: :time_in_state_breakdown,
          params: {}
        }
      ]
    end

    def as_percent?
      options[:as_percent]
    end

    def time_in_state_breakdown
      time_in_state = time_buckets(start_time, end_time, timeframe, bucket_default_val: {
        door_open: 0,
        door_closed: 0,
        engineering_lockout: 0
      })
      EerKpiCalculator.timeline_calculator(start_time, end_time, machine_ids, true) do |timeline, event_indices_within_timeline, shift_start_time, shift_end_time|

        next if event_indices_within_timeline.empty?

        # Loop over each event and add duration to hash
        event_indices_within_timeline.each do |index|
          event = timeline[index]
          duration = EerKpiCalculator.event_duration_in_shift(event, shift_start_time, shift_end_time)
          if timeframe_all?
            time_in_state[event.event_type.to_sym] += duration
          else
            time_in_state[time_bucket_index(shift_start_time)][event.event_type.to_sym] += duration
          end
        end
      end

      time_in_state
    end

    def build_results
      buckets = data[:time_in_state_breakdown]

      # Maybe add a method in hub to get those activeTime between multiple intervals
      if timeframe_all?
        if Rails.env.test? || RxModule::Communicator::Shifts.shifts_between(start_time, end_time)[:models].blank?
          time_in_shift = (end_time - start_time)
        else
          time_in_shift = RxModule::Communicator::Shifts.active_time_between(start_time, end_time) * machine_ids.size
        end
        buckets[:active_time] = time_in_shift * machine_ids.size
      else
        buckets.each do |time, value|
          end_of_bucket = time + timeframe_length
          if Rails.env.test? || RxModule::Communicator::Shifts.shifts_between(time, end_of_bucket)[:models].blank?
            time_in_shift = timeframe_length
          else
            time_in_shift = RxModule::Communicator::Shifts.active_time_between(time, end_of_bucket) * machine_ids.size
          end
          active_time = time_in_shift * machine_ids.size
          buckets[time][:active_time] = active_time
        end
      end
      buckets
    end

    def percent_buckets(row, m_count)
      if timeframe_all?
        lockout_percent = 0
        open_percent = 0
        closed_percent = 0
        unless row[:active_time][:count] == 0
          lockout_percent = Tools.percentage_maker(row[:engineering_lockout][:count], row[:active_time][:count])
          open_percent = Tools.percentage_maker(row[:door_open][:count], row[:active_time][:count])
          closed_percent = Tools.percentage_maker(row[:door_closed][:count], row[:active_time][:count])
        end
        row[:engineering_lockout][:percent] = lockout_percent
        row[:door_open][:percent] = open_percent
        row[:door_closed][:percent] = closed_percent
      else
        row.each do |time, value_hash|
          lockout_percent = 0
          open_percent = 0
          closed_percent = 0
          unless row[time][:active_time][:count] == 0
            lockout_percent = Tools.percentage_maker(value_hash[:engineering_lockout][:count], value_hash[:active_time][:count])
            open_percent = Tools.percentage_maker(value_hash[:door_open][:count], value_hash[:active_time][:count])
            closed_percent = Tools.percentage_maker(value_hash[:door_closed][:count], value_hash[:active_time][:count])
          end
          row[time][:engineering_lockout][:percent] = lockout_percent
          row[time][:door_open][:percent] = open_percent
          row[time][:door_closed][:percent] = closed_percent
        end
      end
    end

    def build_results_per_machine
      data[:time_in_state_breakdown]
    end
  end
end
