module MachineKpiCalculator
  class RoutingKickoutBreakdown < MachineKpiCalculator
    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {top_5: false, sum: false})
        kpi = RoutingKickoutBreakdown.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def top_5?
      !!options[:top_5]
    end

    def handlers
      [
        {
          key: :kickouts,
          handler: KpiTableHandler::RoutingKickoutsKpiHandler,
          params: {
          }
        }
      ]
    end

    # This should return same top5 errors per machine in buckets
    def build_results
      data[:kickouts] unless top_5?
      total_kickout_count_hash = {}
      if timeframe_all?
        total_kickout_count_hash = data[:kickouts]
      else
        data[:kickouts].each do |_, kickout_hash|
          total_kickout_count_hash.merge!(kickout_hash) do|k, val1 , val2|
            val1 + val2
          end
        end
      end
      sorting_array = total_kickout_count_hash.select {|k,v| v > 0}.map {|k, v| {kickout_key: k, count: v}}
      sorting_array.sort! {|a,b| b[:count] <=> a[:count]}
      top_5_kickout_names = sorting_array.slice(0,5).map {|e| e[:kickout_key]}
      results = {}
      if timeframe_all?
        top_5_kickout_names.each do |name|
          results[name] = data[:kickouts][name]
        end
      else
        data[:kickouts].each do |time, kickout_hash|
          results[time] = top_5_kickout_names.zip(top_5_kickout_names.map do |name|
            kickout_hash[name]
          end).to_h
        end
      end
      results
    end

    def build_results_per_machine
      return data[:kickouts] unless top_5?
      total_kickout_count_hash = {}
      data[:kickouts].each do |machine_id, buckets|
        buckets.each do |time, kickout_hash|
          total_kickout_count_hash.merge!(kickout_hash) do|k, val1 , val2|
            val1 + val2
          end
        end
        total_kickout_count_hash
      end
      sorting_array = total_kickout_count_hash.map {|k, v| {kickout_key: k, count: v}}
      sorting_array.sort! {|a,b| b[:count] <=> a[:count]}
      top_5_kickout_names = sorting_array.slice(0,5).map {|e| e[:kickout_key]}
      results = {}
      data[:kickouts].each do |machine_id, buckets|
        results[machine_id] ||= {}
        buckets.each do |time, kickout_hash|
          results[machine_id][time] = top_5_kickout_names.zip(top_5_kickout_names.map do |name|
            kickout_hash[name]
          end).to_h
        end
      end
      results
    end
  end
end

