# == Schema Information
#
# Table name: blocks
#
#  id                   :bigint           not null, primary key
#  cycles_until_discard :integer          default(500)
#  cycles_until_wash    :integer          default(30)
#  dmc                  :string(4000)
#  rfid_tag             :string(4000)
#  side                 :integer
#  usage_count          :integer          default(0)
#  created_at           :datetime         not null
#  updated_at           :datetime         not null
#  job_id               :bigint
#
# Indexes
#
#  index_blocks_on_dmc     (dmc)
#  index_blocks_on_job_id  (job_id)
#
# Foreign Keys
#
#  fk_rails_...  (job_id => jobs.id) ON DELETE => nullify
#
class Block < ApplicationRecord
  include Pushable
  include Sideable

  belongs_to :job, required: false, touch: true
  has_many :block_reactivate_histories, dependent: :destroy

  def self.get_by_dmc(dmc)
    block = Block.find_by(dmc: dmc)
    if block.nil?
      block = Block.create(dmc: dmc)
      settings = BlockReuseSetting.first
      block.cycles_until_wash = settings.block_wash_after
      block.cycles_until_discard = settings.block_discard_after
    end
    block
  end

  def increase_use_count(cycles)
    increment :usage_count
    decrement :cycles_until_discard, cycles

    # If the new wash cycle would force the block to be washed, assign to block the new value minus the difference (essentially assume the operator washed it)
    if cycles_until_wash - cycles < 0
      settings = BlockReuseSetting.first
      increment :cycles_until_wash, settings.block_wash_after - cycles
    else
      decrement :cycles_until_wash, cycles
    end
  end
end
