# == Schema Information
#
# Table name: machine_groups
#
#  id         :bigint           not null, primary key
#  color      :string(4000)
#  name       :string(4000)
#  created_at :datetime         not null
#  updated_at :datetime         not null
#
class MachineGroup < ApplicationRecord
  has_and_belongs_to_many :machines
  has_many :error_filters, as: :filter_target
  has_many :abort_filters, as: :filter_target

  def self.eer_machine_groups
    groups_with_non_eer_machines = MachineGroup.joins(:machines).where(machines: {track_eer_metrics: false}).uniq.pluck(:id)
    MachineGroup.joins(:machines).where.not(machine_groups: {id: groups_with_non_eer_machines}).uniq
  end

  def self.blocker_machine_groups
    groups_with_non_blocker_machines = MachineGroup.joins(machines: :machine_type).where.not(machines: { machine_types: {category: 'blocker'}}).uniq.pluck(:id)
    MachineGroup.joins(:machines).where.not(machine_groups: {id: groups_with_non_blocker_machines}).uniq
  end

  def self.reject_machine_groups
    all_reject_kickout_ids = Machine.joins(:machine_reject_machines).uniq.pluck(:id)
    candidates = MachineGroup.joins(:machines).where(machines: {id: all_reject_kickout_ids}).uniq.pluck(:id)
    not_candidates = MachineGroup.joins(:machines).where.not(machines: {id: all_reject_kickout_ids}).uniq.pluck(:id)
    candidates.reject! {|id| not_candidates.include? id}
    MachineGroup.where(id: candidates)
  end
end
