module ReportMachineGroups
  extend ActiveSupport::Concern

  class InvalidMachineGroupTarget < StandardError; end

  def resolve_machine_group
    raise InvalidMachineGroupTarget if @parameters[:machine_group_ids].blank?
    [@parameters[:machine_group_ids].to_i]
  end

  def resolve_machine_targets
    [MachineGroup.find_by(id: resolve_machine_group).name]
  end

  class_methods do
    def use_machine_group_targets(multiple)
      parameter(
        :machine_group_ids,
        "Machine Group",
        "The machine group that will be the target of the report",
        :string,
        multiple: multiple,
        required: true,
        value_list: -> {machine_group_targets})
    end

    def machine_group_targets
      (MachineGroup.all.map do |g|
        next if g.machines.length < 1
        { label: g.name, value: g.id, group: "Machine Group", description: "Report Kickout values for the entire #{g.name} group" }
      end).compact
    end
  end
end
