module ReportComponents
  class MachinePerformanceOverview < ReportComponent
    include DataHelpers
    include ReportComponentHelpers
    include KpiHelpers

    class InvalidTimeFrame < StandardError; end

    def component_id
      'machine_performance_overview'
    end

    def component_type
      'bar-line'
    end

    def generate(args)
      # Because we are going to be stacking the utilization and uptime and that uptime contains utilization, we are going
      machine_states = MachineKpi.state_breakdown_between(start_time, end_time, timeframe, machine_ids, options: {addons: [:as_percent]})
      waitingforjobs = extract_target_job_state(machine_states, :waitingforjobs)
      processing = extract_target_job_state(machine_states, :processing)

      throughput = MachineKpi.throughput_between(start_time, end_time, timeframe, machine_ids)

      {
        options: {
          metric: ProductionSetting.instance.throughput_metric.capitalize,
          targets: {
            uptime: args[:uptime_target],
            utilization: args[:utilization_target],
          }
        },
        series: [
          build_series_for_kpi_bucket('processing', processing, 'bar', :format_processing),
          build_series_for_kpi_bucket('waitingforjobs', waitingforjobs, 'bar', :format_waitingforjobs),
          build_series_for_kpi_bucket('throughput', throughput, 'line', :format_throughput)
        ]
      }
    end

    def format_processing(bucket_value)
      bucket_value[:processing][:percent]
    end

    def format_waitingforjobs(bucket_value)
      bucket_value[:waitingforjobs][:percent]
    end

    def format_throughput(bucket_value)
      bucket_value[:count]
    end

    private

    # Should always be Timeframe::ALL so always one bucket?
    def extract_target_job_state(data, target_state)
      return data[target_state] if timeframe_all?

      new_data = {}
      data.each do |time_bucket, data|
        new_data[time_bucket] = data.slice(target_state)
      end
      new_data
    end
  end
end

