module ReportComponents
  class TopKickoutTrend < ReportComponent
    include DataHelpers
    include ReportComponentHelpers

    def component_id
      'top_kickouts_trend'
    end

    def component_type
      'line'
    end

    def generate(args)
      @top_5_kickouts = MachineKpi.top_5_kickouts_between(start_time, end_time, timeframe, machine_ids)

      return set_no_data('No kickouts during reporting period') if @top_5_kickouts.all? {|k,v| v.empty?}

      job_state_machine_ids = Machine.where(id: machine_ids).map {|reject_machine| reject_machine.machine_ids}.flatten.uniq
      @job_states = MachineKpi.started_between(start_time, end_time, timeframe, job_state_machine_ids)

      {
        series: build_series,
        colors: color_for_series(5)
      }
    end

    def build_series
      series = {}
      @top_5_kickouts.map do |time, hash|
        hash.each do |reason, value|
          series[reason] ||= {data:[], name: reason}
          series[reason][:data] << {
            y: y_calc(value[:count], @job_states[time][:count]),
            x: bucket_time_to_timeframe_label(time)
          }
        end
      end
      series.values
    end

    def y_calc(numerator, denominator)
      if denominator.nil? && numerator > 0
        y = 100.0
      elsif denominator.nil? || denominator == 0
        y = 0.0
      else
        y = (numerator.to_f/denominator.to_f) * 100
      end
      y.round(1)
    end
  end
end

