class AddExpiryDateToJobs < ActiveRecord::Migration[6.0]
  def up
    add_column :jobs, :expiry_date, :datetime

    # Due to a bug in migration with sharded DBS, we cannot use active record before an exec call, so get the expiry window manually
    settings = exec_query('select job_max_age from settings where id=1').entries[0]
    expiry_window = settings.nil? ? 336 : settings["job_max_age"]

    #Job.reset_column_information
    #expiry_window = Setting.first&.job_max_age || 12

    sql = case ActiveRecord::Base.connection_config[:adapter].downcase
          when "postgres", "postgresql", "postgis"
            <<~SQL
              UPDATE jobs
              SET expiry_date = created_at + INTERVAL '#{expiry_window} hour'
            SQL
          else
            <<~SQL
              UPDATE jobs
              SET expiry_date = DATEADD(hour, #{expiry_window}, created_at)
            SQL
    end
    execute sql
  end

  def down
    remove_column :jobs, :expiry_date, :datetime
  end
end
